@@ -1,10 +1,10 @@
 /*
-    Argument
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    argument
+    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "argument.h"
 #include "misc.h"
@@ -181,8 +180,8 @@ arg_analyse (gchar              **argv,
                   arg_exit = arg->data;
                   g_free (arg_files);
                   if (arg_exit->str)
-                    g_print (arg_exit->str);
-                  gtk_exit (0);
+                    g_print ("%s", arg_exit->str);
+                  exit (0);
                 }
                 break;
               case ARGUMENT_TYPE_HELP:
@@ -230,7 +229,7 @@ arg_analyse (gchar              **argv,
                       g_free (tmp1);
                     }
                   g_free (format);
-                  gtk_exit (0);
+                  exit (0);
                 }
             }
         }
