@@ -1,6 +1,6 @@
 /*
     fileio
-    copyright (c) 1998-2010 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ fileio_get_absolute_path (const gchar *file,
 # if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR p, lpszFile;
   TCHAR szFile[MAX_PATH];   /* ja:フルパス */
-  TCHAR lpszLongName[MAX_PATH];
+  TCHAR szLongName[MAX_PATH];
 # endif /* not GLIB_CHECK_VERSION(2,6,0) */
 
   if (!file)
@@ -165,7 +165,7 @@ fileio_get_absolute_path (const gchar *file,
                 FindClose (hFind);
               else
                 lstrcpyW (rcf.cFileName, L".");
-              if (lstrcmpW (rcf.cFileName, L"."))
+              if (lstrcmpW (rcf.cFileName, L".") != 0)
                 {
                   g_memmove (szLongName + s, rcf.cFileName,
                                     lstrlenW (rcf.cFileName) * sizeof (WCHAR));
@@ -192,29 +192,15 @@ fileio_get_absolute_path (const gchar *file,
       CHAR szFile[MAX_PATH];   /* ja:フルパス */
       CHAR szLongName[MAX_PATH];
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
-
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
+
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       if (PathIsRelativeA (lpszFile))
         {
           LPSTR lpszPath = NULL, lpszTemp;
 
           if (path)
-            {
-#  if GLIB_CHECK_VERSION(2,8,0)
-              lpszPath = g_win32_locale_filename_from_utf8 (path);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-              lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-              lpszPath = g_strdup (path);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
-            }
+            lpszPath = g_win32_locale_filename_from_utf8 (path);
           if (!lpszPath || PathIsRelativeA (lpszPath))
             {
               g_free (lpszPath);
@@ -276,7 +262,7 @@ fileio_get_absolute_path (const gchar *file,
                 FindClose (hFind);
               else
                 lstrcpyA (rcf.cFileName, ".");
-              if (lstrcmpA (rcf.cFileName, "."))
+              if (lstrcmpA (rcf.cFileName, ".") != 0)
                 {
                   g_memmove (szLongName + s, rcf.cFileName,
                                     lstrlenA (rcf.cFileName) * sizeof (CHAR));
@@ -556,13 +542,7 @@ fileio_open (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       hFile = CreateFileA (lpszFile, access, share, NULL, mode,
                                                 FILE_ATTRIBUTE_NORMAL, NULL);
       g_free (lpszFile);
@@ -1135,13 +1115,7 @@ fileio_mkdir (const gchar *dir)
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszPath = g_win32_locale_filename_from_utf8 (path);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszPath = g_strdup (path);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       result = CreateDirectoryA (lpszPath, NULL);
       g_free (lpszPath);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
@@ -1213,13 +1187,7 @@ fileio_remove (const gchar *path)
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
               lpszPath = g_win32_locale_filename_from_utf8 (path);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-              lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-              lpszPath = g_strdup (path);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
               result = RemoveDirectoryA (lpszPath);
               g_free (lpszPath);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
@@ -1262,13 +1230,7 @@ fileio_remove (const gchar *path)
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
           lpszPath = g_win32_locale_filename_from_utf8 (path);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-          lpszPath = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-          lpszPath = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
           result = DeleteFileA (lpszPath);
           g_free (lpszPath);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
@@ -1308,7 +1270,7 @@ fileio_gettime (const gchar *file,
 #  endif /* UNICODE */
 # endif /* not GLIB_CHECK_VERSION(2,6,0) */
 #else /* not G_OS_WIN32 */
-  struct stat buf;
+  GStatBuf buf;
 #endif /* not G_OS_WIN32 */
 
   if (!file)
@@ -1348,13 +1310,7 @@ fileio_gettime (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || !defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       if (!lpszFile)
         return FALSE;
       hFind = FindFirstFileA (lpszFile, &rcFind);
@@ -1400,7 +1356,7 @@ fileio_settime (const gchar *file,
 #  endif /* UNICODE */
 # endif /* not GLIB_CHECK_VERSION(2,6,0) */
 #else /* not G_OS_WIN32 */
-  struct stat sbuf;
+  GStatBuf sbuf;
   struct utimbuf ubuf;
 #endif /* not G_OS_WIN32 */
 
@@ -1433,13 +1389,7 @@ fileio_settime (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       hFile = CreateFileA (lpszFile, GENERIC_WRITE,
                                 FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
                                 OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
@@ -1544,6 +1494,7 @@ fileio_setsystime (FileIOTime    *ftime,
 * ja:応用ファイル関数群                                                       *
 *                                                                             *
 ******************************************************************************/
+#ifndef USE_GTK_EMULATE
 /*  ja:ファイルのアイコンを取得する
          file,ファイル名
     icon_size,アイコンサイズ
@@ -1615,13 +1566,7 @@ fileio_extract_icon (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,8,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,8,0) */
       if ((BOOL)SHGetFileInfoA (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
                                 SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags))
         hIcon = shfi.hIcon;
@@ -1831,3 +1776,4 @@ fileio_extract_icon (const gchar *file,
   return pixbuf;
 #endif /* defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
 }
+#endif /* not USE_GTK_EMULATE */
