@@ -1,10 +1,10 @@
 /*
     misc
-    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,12 +13,10 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "fileio.h"
 #include "misc.h"
-#include <gdk/gdkkeysyms.h>
 #ifdef G_OS_WIN32
 # include <shlobj.h>
 # include <tchar.h>
@@ -196,18 +194,29 @@ GtkWidget *
 misc_create_toolbar (MiscCreateToolbarEntry *entries)
 {
   gint i;
+#if ! GTK_CHECK_VERSION(2,12,0)
   GtkTooltips *tooltips;
+#endif /* not GTK_CHECK_VERSION(2,12,0) */
   GtkWidget *toolbar;
 
   toolbar = gtk_toolbar_new ();
+#if GTK_CHECK_VERSION(2,16,0)
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (toolbar),
+                                                GTK_ORIENTATION_HORIZONTAL);
+#else /* not GTK_CHECK_VERSION(2,16,0) */
   gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar),
                                                 GTK_ORIENTATION_HORIZONTAL);
+#endif /* not GTK_CHECK_VERSION(2,16,0) */
   gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
+#if ! GTK_CHECK_VERSION(2,14,0)
   gtk_toolbar_set_tooltips (GTK_TOOLBAR (toolbar), TRUE);
+#endif /* not GTK_CHECK_VERSION(2,14,0) */
 #if GTK_CHECK_VERSION(2,4,0)
   gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), FALSE);
 #endif /* GTK_CHECK_VERSION(2,4,0) */
+#if ! GTK_CHECK_VERSION(2,12,0)
   tooltips = gtk_tooltips_new ();
+#endif /* not GTK_CHECK_VERSION(2,12,0) */
   for (i = 0; entries[i].path; i++)
     {
       if (entries[i].name)
@@ -220,8 +229,13 @@ misc_create_toolbar (MiscCreateToolbarEntry *entries)
 #if GTK_CHECK_VERSION(2,4,0)
               entries[i].tool_item = gtk_tool_button_new
                     (gtk_image_new_from_pixbuf (pixbuf), _(entries[i].name));
+# if GTK_CHECK_VERSION(2,12,0)
+              gtk_tool_item_set_tooltip_text (entries[i].tool_item,
+                                                         _(entries[i].name));
+# else /* not GTK_CHECK_VERSION(2,12,0) */
               gtk_tool_item_set_tooltip (entries[i].tool_item, tooltips,
                                     _(entries[i].name), _(entries[i].name));
+# endif /* not GTK_CHECK_VERSION(2,12,0) */
 #else /* not GTK_CHECK_VERSION(2,4,0) */
               entries[i].tool_item = gtk_toolbar_append_item
                                             (GTK_TOOLBAR (toolbar),
@@ -244,8 +258,12 @@ misc_create_toolbar (MiscCreateToolbarEntry *entries)
 #if GTK_CHECK_VERSION(2,4,0)
               entries[i].tool_item = gtk_tool_button_new_from_stock
                                                             (entries[i].name);
+# if GTK_CHECK_VERSION(2,12,0)
+              gtk_tool_item_set_tooltip_text (entries[i].tool_item, text);
+# else /* not GTK_CHECK_VERSION(2,12,0) */
               gtk_tool_item_set_tooltip (entries[i].tool_item, tooltips,
                                                                 text, text);
+# endif /* not GTK_CHECK_VERSION(2,12,0) */
 #else /* not GTK_CHECK_VERSION(2,4,0) */
               entries[i].tool_item = gtk_toolbar_insert_stock
                                                 (GTK_TOOLBAR (toolbar),
@@ -350,7 +368,8 @@ misc_spec_config (const gchar *group,
                     {
                       gchar *dir;
 
-                      dir = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
+                      dir = g_filename_from_utf8
+                                            (utf8str, -1, NULL, NULL, NULL);
                       g_free (utf8str);
                       if (dir)
                         {
@@ -433,9 +452,9 @@ misc_spec_locale (const gchar *prgname)
 #else /* not LOCALEDIR */
   static gchar *locale = NULL;
   gchar *dirname, *path;
-# if GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32)
+# ifdef G_OS_WIN32
   gchar *tmp;
-# endif /* GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
+# endif /* G_OS_WIN32 */
 
   if (locale)
     return locale;
@@ -448,18 +467,18 @@ misc_spec_locale (const gchar *prgname)
   if (!g_file_test (locale, G_FILE_TEST_EXISTS))
     {
       g_free (locale);
+#  if GLIB_CHECK_VERSION(2,16,0)
+      dirname = g_win32_get_package_installation_directory_of_module (NULL);
+      locale = g_build_filename (dirname, "share", "locale", NULL);
+      g_free (dirname);
+#  else /* not GLIB_CHECK_VERSION(2,16,0) */
       locale = g_win32_get_package_installation_subdirectory
-                            (NULL, prgname, "lib" G_DIR_SEPARATOR_S "locale");
+                        (NULL, prgname, "share" G_DIR_SEPARATOR_S "locale");
+#  endif /* not GLIB_CHECK_VERSION(2,16,0) */
     }
-#  if GLIB_CHECK_VERSION(2,6,0)
-#   if GLIB_CHECK_VERSION(2,8,0)
   tmp = g_win32_locale_filename_from_utf8 (locale);
-#   else /* not GLIB_CHECK_VERSION(2,8,0) */
-  tmp = g_locale_from_utf8 (locale, -1, NULL, NULL, NULL);
-#   endif /* not GLIB_CHECK_VERSION(2,8,0) */
   g_free (locale);
   locale = tmp;
-#  endif /* GLIB_CHECK_VERSION(2,6,0) */
 # endif /* G_OS_WIN32 */
   return locale;
 #endif /* not LOCALEDIR */
@@ -643,13 +662,13 @@ misc_mnemonic_to_text (const gchar *mnemonic)
          page,ページ
           pos,位置                                                          */
 void
-misc_set_scroll_bar (GtkWidget     *scroll,
-                     GtkSignalFunc  func,
-                     gpointer       func_data,
-                     const gint     min,
-                     const gint     max,
-                     const gint     page,
-                     const gint     pos)
+misc_set_scroll_bar (GtkWidget  *scroll,
+                     GCallback   func,
+                     gpointer    func_data,
+                     const gint  min,
+                     const gint  max,
+                     const gint  page,
+                     const gint  pos)
 {
   GtkAdjustment *adjust;
 
@@ -677,19 +696,41 @@ misc_scroll_window (GtkWidget  *widget,
                     const gint  width,
                     const gint  height)
 {
-  GdkGC *gc;
-
   if (ABS (dx) > width || ABS (dy) > height)
     {
       gtk_widget_queue_draw_area (widget, x, y, width, height);
     }
   else if (dx != 0 || dy != 0)
     {
-      gc = gdk_gc_new (widget->window);
+#if GTK_CHECK_VERSION(2,24,0)
+      cairo_t *cr;
+      gint w, h, xx, yy;
+      GdkPixbuf *pixbuf;
+      GdkWindow *window;
+
+      window = gtk_widget_get_window (widget);
+      cr = gdk_cairo_create (window);
+      w = width - ABS (dx);
+      h = height - ABS (dy);
+      pixbuf = gdk_pixbuf_get_from_drawable (NULL, window,
+                                    gdk_colormap_get_system (),
+                                    dx > 0 ? x : x - dx, dy > 0 ? y : y - dy,
+                                    0, 0, w, h);
+      xx = dx > 0 ? x + dx : x;
+      yy = dy > 0 ? y + dy : y;
+      gdk_cairo_set_source_pixbuf (cr, pixbuf, xx, yy);
+      g_object_unref (pixbuf);
+      cairo_rectangle (cr, xx, yy, w, h);
+      cairo_fill (cr);
+      cairo_destroy (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
+      GdkGC *gc;
+
+      gc = gdk_gc_new (gtk_widget_get_window (widget));
       gdk_gc_set_exposures (gc, TRUE);
-      gdk_draw_drawable  (widget->window,
+      gdk_draw_drawable  (gtk_widget_get_window (widget),
                           gc,
-                          widget->window,
+                          gtk_widget_get_window (widget),
                           dx > 0 ? x : x - dx,
                           dy > 0 ? y : y - dy,
                           dx > 0 ? x + dx : x,
@@ -697,6 +738,7 @@ misc_scroll_window (GtkWidget  *widget,
                           width - ABS (dx),
                           height - ABS (dy));
       g_object_unref (gc);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
       if (dx != 0)
         gtk_widget_queue_draw_area (widget, dx > 0 ? x : x + width + dx, y,
                                                             ABS (dx), height);
@@ -704,7 +746,7 @@ misc_scroll_window (GtkWidget  *widget,
         gtk_widget_queue_draw_area (widget, x, dy > 0 ? y : y + height + dy,
                                                             width, ABS (dy));
     }
-  gdk_window_process_updates (widget->window, TRUE);
+  gdk_window_process_updates (gtk_widget_get_window (widget), TRUE);
 }
 
 
@@ -714,7 +756,7 @@ misc_dialog_key_press (GtkWidget   *widget,
                        GdkEventKey *event,
                        gpointer     user_data)
 {
-  if (event->keyval == GDK_Escape)
+  if (event->keyval == GDK_KEY_Escape)
     gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_NONE);
   return FALSE;
 }
