@@ -1,6 +1,6 @@
 /*
     profile
-    copyright (c) 1998-2010 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "fileio.h"
-#include "misc.h"
 #include "profile.h"
 
 
@@ -309,10 +308,10 @@ profile_get_string (Profile     *profile,
 
   if (!profile || !section || !key)
     return NULL;
-  for (p = profile->list; p; p = p->next)
+  for (p = profile->list; p; p = p->next){
     if (p->type == PROFILE_DATA_TYPE_KEY && g_strcmp (p->section, section) == 0
                                                 && g_strcmp (p->key, key) == 0)
-      return g_strdup (p->value);
+      return g_strdup (p->value);}
   for (p = profile->sublist; p; p = p->next)
     if (p->type == PROFILE_DATA_TYPE_KEY && g_strcmp (p->section, section) == 0
                                                 && g_strcmp (p->key, key) == 0)
@@ -333,8 +332,6 @@ profile_get_size (Profile     *profile,
                   const gchar *key,
                   const guint  type)
 {
-  guint8 *array;
-  gsize n;
   ProfileList *p;
 
   if (!profile || !section || !key)
@@ -367,11 +364,20 @@ profile_get_size (Profile     *profile,
       case PROFILE_VALUE_TYPE_STRING:
         return g_strlen (p->value) + 1;
       case PROFILE_VALUE_TYPE_ARRAY:
-        array = misc_str_to_array (&n, 8, p->value, 10, FALSE);
-        if (!array)
-          return 0;
-        g_free (array);
-        return n;
+        {
+          gsize n = 0;
+          gchar *nptr, *endptr;
+
+          nptr = endptr = p->value;
+          while (*endptr != '\0')
+            {
+              g_strtoul (nptr, &endptr, 0);
+              if (nptr != endptr)
+                n++;
+              nptr = endptr + 1;
+            }
+          return n;
+        }
     }
   return 0;
 }
@@ -393,8 +399,6 @@ profile_get_value (Profile     *profile,
                    const gsize  size,
                    const guint  type)
 {
-  guint8 *array;
-  gsize n;
   ProfileList *p;
 
   if (!profile || !section || !key || !value)
@@ -431,7 +435,7 @@ profile_get_value (Profile     *profile,
       case PROFILE_VALUE_TYPE_INT:
         if (size < sizeof (gint))
           return FALSE;
-        misc_str_to_val ((gint *)value, p->value, 10, TRUE);
+        *((gint *)value) = g_strtol (p->value, NULL, 0);
         break;
       case PROFILE_VALUE_TYPE_STRING:
         if (size < g_strlen (p->value) + 1)
@@ -439,14 +443,23 @@ profile_get_value (Profile     *profile,
         g_strcpy ((gchar *)value, p->value);
         break;
       case PROFILE_VALUE_TYPE_ARRAY:
-        array = misc_str_to_array (&n, 8, p->value, 10, FALSE);
-        if (!array)
-          return FALSE;
-        if (size <= n)
-          g_memmove (value, array, size);
-        g_free (array);
-        if (n < size)
-          return FALSE;
+        {
+          gchar *nptr, *endptr;
+          guint8 *v;
+          gint n = 0;
+
+          v = value;
+          nptr = endptr = p->value;
+          while (*endptr != '\0' && n < size)
+            {
+              v[n] = g_strtoul (nptr, &endptr, 0);
+              if (nptr != endptr)
+                n++;
+              nptr = endptr + 1;
+            }
+          if (n < size)
+            return FALSE;
+        }
         break;
       default:
         return FALSE;
@@ -562,7 +575,6 @@ profile_set_value (Profile       *profile,
       default:
         return FALSE;
     }
-  p->type = type;
   p->value = g_strchr (p->data, '=') + 1;
   profile->edit = TRUE;
   return TRUE;
@@ -758,6 +770,10 @@ profile_keyfile_load_from_data (Profile      *profile,
   if (data && length != 0)
     {
       if (length == (gsize)-1)
+        {
+          profile->list = profile_list_from_text (data);
+        }
+      else
         {
           gchar *text;
 
@@ -765,10 +781,6 @@ profile_keyfile_load_from_data (Profile      *profile,
           profile->list = profile_list_from_text (text);
           g_free (text);
         }
-      else
-        {
-          profile->list = profile_list_from_text (data);
-        }
     }
   return TRUE;
 }
@@ -1068,6 +1080,8 @@ profile_keyfile_set_string (Profile     *profile,
   gchar *str;
   gint i = 0;
 
+  if (!string)
+    return;
   str = g_malloc ((g_strlen (string) * 4 + 1) * sizeof (gchar));
   str[0] = '\0';
   while (string[i] != '\0')
@@ -1093,9 +1107,9 @@ profile_keyfile_set_string (Profile     *profile,
                                                     || !g_unichar_isprint (c))
                 {
                   tmp[0] = '\\';
-                  tmp[1] = '0' + (((string[i]) >> 6) & 07);
-                  tmp[2] = '0' + (((string[i]) >> 3) & 07);
-                  tmp[3] = '0' + ((string[i]) & 07);
+                  tmp[1] = '0' + ((string[i] >> 6) & 7);
+                  tmp[2] = '0' + ((string[i] >> 3) & 7);
+                  tmp[3] = '0' + (string[i] & 7);
                   tmp[4] = '\0';
                   i++;
                 }
