@@ -1,6 +1,6 @@
 /*
     orzmdi
-    copyright (c) 1998-2010 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -57,34 +57,22 @@ static gint orz_mdi_signals[LAST_SIGNAL] = {0};
 static void
 orz_mdi_label_markup (OrzMdiTab *tab)
 {
-  gchar *label, *str;
+  gchar *label;
 
-  if (tab->same < -1)
-    {
-#if GLIB_CHECK_VERSION(2,6,0)
-      label = g_filename_display_basename (tab->file);
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-      str = g_path_get_basename (tab->file);
-      label = g_filename_to_utf8 (str, -1, NULL, NULL, NULL);
-      g_free (str);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
-    }
-  else
-    {
-#if GLIB_CHECK_VERSION(2,6,0)
-      label = g_filename_display_name (tab->file);
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-      label = g_filename_to_utf8 (tab->file, -1, NULL, NULL, NULL);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
-    }
+  label = tab->same < -1 ? g_filename_display_basename (tab->file)
+                         : g_filename_display_name (tab->file);
   if (tab->edited)
     {
+      gchar *str;
+
       str = g_strconcat ("<b><i>", label, "</i></b>", NULL);
       g_free (label);
       label = str;
     }
   if (tab->same >= 0)
     {
+      gchar *str;
+
       str = g_strdup_printf ("%s:%d", label, tab->same);
       g_free (label);
       label = str;
@@ -268,7 +256,11 @@ orz_mdi_remove (GtkContainer *container,
 
 static void
 orz_mdi_switch_page (GtkNotebook     *notebook,
+#if GTK_CHECK_VERSION(2,22,0)
+                     gpointer         page,
+#else /* not GTK_CHECK_VERSION(2,22,0) */
                      GtkNotebookPage *page,
+#endif /* not GTK_CHECK_VERSION(2,22,0) */
                      guint            page_num)
 {
   GtkWidget *child;
@@ -381,7 +373,9 @@ orz_mdi_class_init (OrzMdiClass *klass)
 static void
 orz_mdi_init (OrzMdi *mdi)
 {
+#if ! GTK_CHECK_VERSION(2,12,0)
   mdi->tooltips = gtk_tooltips_new ();
+#endif /* not GTK_CHECK_VERSION(2,12,0) */
 }
 
 
@@ -487,7 +481,7 @@ orz_mdi_tab_drag_data_get (GtkWidget        *widget,
   const static gchar *text = "dummy";
 
   if (info == TARGET_ORZ_MDI_TAB)
-    gtk_selection_data_set (data, data->target, 8,
+    gtk_selection_data_set (data, gtk_selection_data_get_selection (data), 8,
                                         (const guchar *)text, g_strlen (text));
 }
 static void
@@ -618,7 +612,11 @@ orz_mdi_insert_page (OrzMdi         *mdi,
   gtk_event_box_set_visible_window (GTK_EVENT_BOX (event), FALSE);
 #endif /* GLIB_CHECK_VERSION(2,4,0) */
   gtk_container_add (GTK_CONTAINER (event), hbox);
+#if GTK_CHECK_VERSION(2,12,0)
+  gtk_widget_set_has_tooltip (event, TRUE);
+#else /* not GTK_CHECK_VERSION(2,12,0) */
   gtk_tooltips_set_tip (mdi->tooltips, event, tab->file, tab->file);
+#endif /* not GTK_CHECK_VERSION(2,12,0) */
   g_object_set_data (G_OBJECT (event), "user_data", tab);
 
   g_signal_connect (G_OBJECT (button), "clicked",
@@ -628,10 +626,10 @@ orz_mdi_insert_page (OrzMdi         *mdi,
   g_signal_connect (G_OBJECT (event), "destroy",
                                     G_CALLBACK (orz_mdi_destroy_tab), mdi);
   /* en:Drag & Drop */
-  gtk_drag_source_set (event, GDK_BUTTON1_MASK, dnd_entries, 1,
-                                                            GDK_ACTION_MOVE);
-  gtk_drag_dest_set (event, GTK_DEST_DEFAULT_ALL, dnd_entries, 1,
-                                                            GDK_ACTION_MOVE);
+  gtk_drag_source_set (event, GDK_BUTTON1_MASK,
+                    dnd_entries, G_N_ELEMENTS (dnd_entries), GDK_ACTION_MOVE);
+  gtk_drag_dest_set (event, GTK_DEST_DEFAULT_ALL,
+                    dnd_entries, G_N_ELEMENTS (dnd_entries), GDK_ACTION_MOVE);
   g_signal_connect (G_OBJECT (event), "drag_begin",
                             G_CALLBACK (orz_mdi_tab_drag_begin), NULL);
   g_signal_connect (G_OBJECT (event), "drag_data_get",
@@ -715,8 +713,8 @@ orz_mdi_get_tab (OrzMdi     *mdi,
           GtkWidget *tab_label;
 
           tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (mdi), child);
-          if (tab_label){
-            return g_object_get_data (G_OBJECT (tab_label), "user_data");}
+          if (tab_label)
+            return g_object_get_data (G_OBJECT (tab_label), "user_data");
         }
     }
   return NULL;
@@ -815,8 +813,12 @@ orz_mdi_set_file (OrzMdi      *mdi,
                   file_add_edit (mdi, tab);
                   orz_mdi_label_markup (tab);
                   /* ja:ツールチップス */
+# if GTK_CHECK_VERSION(2,12,0)
+                  gtk_widget_set_tooltip_text (tab_label, tab->file);
+# else /* not GTK_CHECK_VERSION(2,12,0) */
                   gtk_tooltips_set_tip (mdi->tooltips, tab_label,
                                                         tab->file, tab->file);
+# endif /* not GTK_CHECK_VERSION(2,12,0) */
                   /* ja:アイコン */
                   hbox = gtk_bin_get_child (GTK_BIN (tab_label));
                   if (tab->image)
