@@ -1,10 +1,10 @@
 /*
     orzwinmenu
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,48 +13,16 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "orzwinmenu.h"
 
 
-static void orz_winmenu_class_init (OrzWinmenuClass *klass);
-static void orz_winmenu_init       (OrzWinmenu      *winmenu);
-static void orz_winmenu_dispose    (GObject         *object);
-
-
-static GObjectClass *parent_class = NULL;
-
-
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
-GType
-orz_winmenu_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type)
-    {
-      const static GTypeInfo info =
-      {
-        sizeof (OrzWinmenuClass),
-        NULL,               /* base_init */
-        NULL,               /* base_finalize */
-        (GClassInitFunc)orz_winmenu_class_init,
-        NULL,               /* class_finalize */
-        NULL,               /* class_data */
-        sizeof (OrzWinmenu),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc)orz_winmenu_init,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, "OrzWinmenu", &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (OrzWinmenu, orz_winmenu, G_TYPE_OBJECT);
+static void orz_winmenu_dispose (GObject *object);
 
 
 static void
@@ -62,9 +30,7 @@ orz_winmenu_class_init (OrzWinmenuClass *klass)
 {
   GObjectClass *object_class;
 
-  parent_class = g_type_class_peek_parent (klass);
-  object_class = (GObjectClass *)klass;
-
+  object_class = G_OBJECT_CLASS (klass);
   object_class->dispose = orz_winmenu_dispose;
 }
 
@@ -118,12 +84,7 @@ orz_winmenu_add (OrzMdi     *mdi,
 
       glist = gtk_container_get_children (GTK_CONTAINER (winmenu->menu_shell));
       length = g_list_length (glist);
-#if GLIB_CHECK_VERSION(2,6,0)
       label = g_filename_display_name (orz_mdi_get_file (mdi, page_num));
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-      label = g_filename_to_utf8 (orz_mdi_get_file (mdi, page_num), -1,
-                                                            NULL, NULL, NULL);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
       menu_item = glist ? gtk_radio_menu_item_new_with_label_from_widget
                                     (GTK_RADIO_MENU_ITEM (glist->data), label)
                         : gtk_radio_menu_item_new_with_label (NULL, label);
@@ -205,25 +166,16 @@ orz_winmenu_change (OrzMdi     *mdi,
   if (winmenu && winmenu->mdi == mdi && winmenu->menu_shell && winmenu->glist)
     {
       gchar *label;
-      gint i;
+      GList *glist;
       GtkWidget *child;
 
       child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (mdi), page_num);
-#if GLIB_CHECK_VERSION(2,6,0)
       label = g_filename_display_name (orz_mdi_get_file (mdi, page_num));
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-      label = g_filename_to_utf8 (orz_mdi_get_file (mdi, page_num), -1,
-                                                            NULL, NULL, NULL);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
-      for (i = g_list_length (winmenu->glist) - 1; i >= 0; i--)
-        {
-          GtkWidget *menu_item;
-
-          menu_item = g_list_nth_data (winmenu->glist, i);
-          if (g_object_get_data (G_OBJECT (menu_item), "user_data") == child)
-            gtk_label_set_text (GTK_LABEL (gtk_bin_get_child
-                                                (GTK_BIN (menu_item))), label);
-        }
+      for (glist = g_list_first (winmenu->glist);
+                                            glist; glist = g_list_next (glist))
+        if (g_object_get_data (G_OBJECT (glist->data), "user_data") == child)
+          gtk_label_set_text (GTK_LABEL (gtk_bin_get_child
+                                            (GTK_BIN (glist->data))), label);
       g_free (label);
     }
 }
@@ -237,13 +189,14 @@ orz_winmenu_reorder (OrzMdi     *mdi,
 {
   if (winmenu && winmenu->mdi == mdi && winmenu->menu_shell && winmenu->glist)
     {
-      gint i;
+      GList *glist;
       GtkWidget *child, *menu_item;
 
       child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (mdi), page_num);
-      for (i = g_list_length (winmenu->glist) - 1; i >= 0; i--)
+      for (glist = g_list_first (winmenu->glist);
+                                            glist; glist = g_list_next (glist))
         {
-          menu_item = g_list_nth_data (winmenu->glist, i);
+          menu_item = glist->data;
           if (g_object_get_data (G_OBJECT (menu_item), "user_data") == child)
             {
               g_object_ref (G_OBJECT (menu_item));
@@ -253,10 +206,9 @@ orz_winmenu_reorder (OrzMdi     *mdi,
               break;
             }
         }
-      if (i >= 0)
+      if (glist)
         {
-          gint length, position;
-          GList *glist;
+          gint i, length, position;
 
           glist = gtk_container_get_children
                                         (GTK_CONTAINER (winmenu->menu_shell));
@@ -297,13 +249,16 @@ orz_winmenu_remove (OrzMdi     *mdi,
 {
   if (winmenu && winmenu->mdi == mdi && winmenu->menu_shell && winmenu->glist)
     {
-      gint i;
-      GtkWidget *child, *menu_item;
+      GList *glist;
+      GtkWidget *child;
 
       child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (mdi), page_num);
-      for (i = g_list_length (winmenu->glist) - 1; i >= 0; i--)
+      for (glist = g_list_first (winmenu->glist);
+                                            glist; glist = g_list_next (glist))
         {
-          menu_item = g_list_nth_data (winmenu->glist, i);
+          GtkWidget *menu_item;
+
+          menu_item = glist->data;
           if (g_object_get_data (G_OBJECT (menu_item), "user_data") == child)
             {
               gtk_container_remove (GTK_CONTAINER (winmenu->menu_shell),
@@ -328,12 +283,12 @@ orz_winmenu_remove (OrzMdi     *mdi,
             }
           if (GTK_IS_MENU (winmenu->menu_shell))
             {
-              GList *glist;
-
               glist = gtk_container_get_children
                                         (GTK_CONTAINER (winmenu->menu_shell));
               if (g_list_length (glist) <= 0)
                 {
+                  GtkWidget *menu_item;
+
                   menu_item = gtk_menu_get_attach_widget
                                             (GTK_MENU (winmenu->menu_shell));
                   if (menu_item)
@@ -354,22 +309,20 @@ orz_winmenu_switch (OrzMdi     *mdi,
 {
   if (winmenu && winmenu->mdi == mdi && winmenu->menu_shell && winmenu->glist)
     {
-      gint i;
-      GtkWidget *child, *menu_item;
+      GList *glist;
+      GtkWidget *child;
 
       child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (mdi), page_num);
-      for (i = g_list_length (winmenu->glist) - 1; i >= 0; i--)
-        {
-          menu_item = g_list_nth_data (winmenu->glist, i);
-          if (g_object_get_data (G_OBJECT (menu_item), "user_data") == child)
-            {
-              g_object_set_data (G_OBJECT (menu_item), "user_data", NULL);
-              gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item),
-                                                                        TRUE);
-              g_object_set_data (G_OBJECT (menu_item), "user_data", child);
-              break;
-            }
-        }
+      for (glist = g_list_first (winmenu->glist);
+                                            glist; glist = g_list_next (glist))
+        if (g_object_get_data (G_OBJECT (glist->data), "user_data") == child)
+          {
+            g_object_set_data (G_OBJECT (glist->data), "user_data", NULL);
+            gtk_check_menu_item_set_active
+                                    (GTK_CHECK_MENU_ITEM (glist->data), TRUE);
+            g_object_set_data (G_OBJECT (glist->data), "user_data", child);
+            break;
+          }
     }
 }
 
@@ -377,8 +330,9 @@ orz_winmenu_switch (OrzMdi     *mdi,
 static void
 orz_winmenu_dispose (GObject *object)
 {
-  OrzWinmenu *winmenu = ORZ_WINMENU (object);
+  OrzWinmenu *winmenu;
 
+  winmenu = ORZ_WINMENU (object);
   if (winmenu->mdi)
     {
       gint i;
@@ -412,8 +366,8 @@ orz_winmenu_dispose (GObject *object)
                                                 winmenu->handler_destroy_menu);
       winmenu->menu_shell = NULL;
     }
-  if (G_OBJECT_CLASS (parent_class)->dispose)
-    G_OBJECT_CLASS (parent_class)->dispose (object);
+  if (G_OBJECT_CLASS (orz_winmenu_parent_class)->dispose)
+    G_OBJECT_CLASS (orz_winmenu_parent_class)->dispose (object);
 }
 
 
@@ -457,7 +411,7 @@ orz_winmenu_destroy_menu_shell (GtkWidget  *widget,
 ******************************************************************************/
 /*  ja:新規作成
     RET,オブジェクト                                                        */
-GObject*
+GObject *
 orz_winmenu_new (void)
 {
   return g_object_new (ORZ_TYPE_WINMENU, NULL);
