@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -66,8 +66,10 @@ ftype_copy (FileType *ft_dst,
   g_free (ft_dst->ext);
   ft_dst->ext = ft_src->ext ? g_strdup (ft_src->ext) : NULL;
   g_memmove (ft_dst->color, ft_src->color, sizeof (GdkColor) * 12);
+#if ! GTK_CHECK_VERSION(3,0,0)
   gdk_colormap_alloc_colors (gdk_colormap_get_system (),
                                     ft_dst->color, 12, FALSE, TRUE, success);
+#endif /* not GTK_CHECK_VERSION(3,0,0) */
 }
 
 
@@ -325,9 +327,15 @@ ftype_dialog_toggled (GtkToggleButton *togglebutton,
 
 
 /* ja:ダイアログが破棄されるとき */
+#if GTK_CHECK_VERSION(3,0,0)
+static void
+ftype_dialog_destroy (GtkWidget      *widget,
+                      FileTypeDialog *ftypedlg)
+#else /* not  GTK_CHECK_VERSION(3,0,0) */
 static void
 ftype_dialog_destroy (GtkObject      *object,
                       FileTypeDialog *ftypedlg)
+#endif /* not  GTK_CHECK_VERSION(3,0,0) */
 {
   gint i;
   GtkTreeIter iter;
@@ -501,8 +509,8 @@ ftype_dialog (GtkWindow *parent,
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
                                     GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   /* ja:ボックス */
-  vbox = gtk_vbox_new (FALSE, SPACING);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
+  vbox = gtk_dialog_get_content_area (GTK_DIALOG (ftypedlg.dialog));
+  gtk_box_set_spacing (GTK_BOX (vbox), SPACING);
   gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
   if (ftypedlg.check)
     gtk_box_pack_start (GTK_BOX (vbox), ftypedlg.check, FALSE, FALSE, 0);
@@ -513,7 +521,6 @@ ftype_dialog (GtkWindow *parent,
   gtk_box_pack_start (GTK_BOX (hbox), ftypedlg.button_down, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), ftypedlg.button_edit, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (ftypedlg.dialog)->vbox), vbox);
 
   /* ja:表示 */
   gtk_widget_set_sensitive (ftypedlg.button_add, FALSE);
