@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -47,7 +47,8 @@ hide_caret (TmaidWindow *tmaid)
                                         - tmaid->top.x) * tmaid->font_width,
         (tmaid->cursor.y - tmaid->top.y) * tmaid->font_height,
                                         tmaid->font_width, tmaid->font_height);
-      gdk_window_process_updates (tmaid->drawing->window, TRUE);
+      gdk_window_process_updates (gtk_widget_get_window (tmaid->drawing),
+                                                                        TRUE);
     }
 }
 
@@ -105,7 +106,7 @@ draw_caret (TmaidWindow *tmaid,
   gtk_statusbar_push (GTK_STATUSBAR (status), context_id, text);
   g_free (text);
 #ifndef G_OS_WIN32
-  gdk_window_get_origin (tmaid->drawing->window, &x, &y);
+  gdk_window_get_origin (gtk_widget_get_window (tmaid->drawing), &x, &y);
   widget = gtk_widget_get_parent (tmaid->preedit);
   visible = GTK_WIDGET_VISIBLE (widget);
   if (!visible)
@@ -117,7 +118,7 @@ draw_caret (TmaidWindow *tmaid,
 #endif /* not G_OS_WIN32 */
   rc.width = 0;
   gtk_im_context_set_cursor_location (tmaid->im_context, &rc);
-  gdk_window_process_updates (tmaid->drawing->window, TRUE);
+  gdk_window_process_updates (gtk_widget_get_window (tmaid->drawing), TRUE);
 }
 
 
@@ -253,13 +254,16 @@ clear_sel (TmaidWindow *tmaid,
   else
     {
       /* ja:違う行 */
+      GtkAllocation allocation;
+
+      gtk_widget_get_allocation (tmaid->drawing, &allocation);
       x = 0;
       y = (MIN (start->y, end->y) - tmaid->top.y) * tmaid->font_height;
-      width = tmaid->drawing->allocation.width;
+      width = allocation.width;
       height = (ABS (start->y - end->y) + 1) * tmaid->font_height;
     }
   gtk_widget_queue_draw_area (tmaid->drawing, x, y, width, height);
-  gdk_window_process_updates (tmaid->drawing->window, TRUE);
+  gdk_window_process_updates (gtk_widget_get_window (tmaid->drawing), TRUE);
 }
 
 
@@ -271,12 +275,17 @@ move_edit_window (TmaidWindow *tmaid,
                   GdkPoint    *top)
 {
   if (tmaid->top.x != top->x || tmaid->top.y != top->y)
-    misc_scroll_window (tmaid->drawing,
+    {
+      GtkAllocation allocation;
+
+      gtk_widget_get_allocation (tmaid->drawing, &allocation);
+      misc_scroll_window (tmaid->drawing,
                                 (top->x - tmaid->top.x) * tmaid->font_width,
                                 (top->y - tmaid->top.y) * tmaid->font_height,
                                 0, 0,
-                                tmaid->drawing->allocation.width,
-                                tmaid->drawing->allocation.height);
+                                allocation.width,
+                                allocation.height);
+    }
 }
 
 
@@ -296,14 +305,16 @@ edit_operation (TmaidWindow    *tmaid,
 {
   gint sx, sy, max;
   GdkPoint top;
+  GtkAllocation allocation;
   TmaidHistory *d;
 
   hide_caret (tmaid);
   tmaid->cursor.x = edit_get_align_pos (tmaid,
                                     tmaid->cursor.x, tmaid->cursor.y, FALSE);
   top = tmaid->top;
-  sx = MAX (tmaid->drawing->allocation.width / tmaid->font_width, 1);
-  sy = MAX (tmaid->drawing->allocation.height / tmaid->font_height, 1);
+  gtk_widget_get_allocation (tmaid->drawing, &allocation);
+  sx = MAX (allocation.width / tmaid->font_width, 1);
+  sy = MAX (allocation.height / tmaid->font_height, 1);
   d = g_malloc (sizeof (TmaidHistory));
   if (tmaid->select.x < 0)
     {
@@ -340,13 +351,13 @@ edit_operation (TmaidWindow    *tmaid,
       i = (tmaid->cursor.x - tmaid->top.x - 1) * tmaid->font_width;
       gtk_widget_queue_draw_area (tmaid->drawing,
                     i, (tmaid->cursor.y - tmaid->top.y) * tmaid->font_height,
-                    tmaid->drawing->allocation.width - i, tmaid->font_height);
+                    allocation.width - i, tmaid->font_height);
       /* ja:減った行数分だけスクロール */
       misc_scroll_window (tmaid->drawing, 0, -delete * tmaid->font_height,
         0,
         (tmaid->cursor.y - tmaid->top.y + 1) * tmaid->font_height,
-        tmaid->drawing->allocation.width,
-        MAX (tmaid->drawing->allocation.height
+        allocation.width,
+        MAX (allocation.height
             - (tmaid->cursor.y - tmaid->top.y + 1) * tmaid->font_height, 0));
         /* ja:削除を終えた位置から右マージンの影響のある行数だけ初期化 */
       for (i = 0,
@@ -357,7 +368,7 @@ edit_operation (TmaidWindow    *tmaid,
       if (i > 0)
         gtk_widget_queue_draw_area (tmaid->drawing,
                 0, (tmaid->cursor.y - tmaid->top.y + 1) * tmaid->font_height,
-                    tmaid->drawing->allocation.width, i * tmaid->font_height);
+                    allocation.width, i * tmaid->font_height);
     }
   if (length <= 0 || !text)
     {
@@ -400,13 +411,13 @@ edit_operation (TmaidWindow    *tmaid,
       gtk_widget_queue_draw_area (tmaid->drawing,
                             (cursor_old.x - tmaid->top.x) * tmaid->font_width,
                             (cursor_old.y - tmaid->top.y) * tmaid->font_height,
-                        tmaid->drawing->allocation.width, tmaid->font_height);
+                            allocation.width, tmaid->font_height);
       /* ja:増えた行数分だけスクロール */
       misc_scroll_window (tmaid->drawing, 0, put * tmaid->font_height,
             0,
             (cursor_old.y - tmaid->top.y + 1) * tmaid->font_height,
-            tmaid->drawing->allocation.width,
-            MAX (tmaid->drawing->allocation.height
+            allocation.width,
+            MAX (allocation.height
                 - (cursor_old.y - tmaid->top.y + 1) * tmaid->font_height, 0));
       /* ja:挿入を終えた位置から右マージンの影響のある行数だけ初期化 */
       p = q = edit_get_line_buf (&tmaid->start, &tmaid->off, cursor_new.y);
@@ -418,7 +429,7 @@ edit_operation (TmaidWindow    *tmaid,
       if (j > 0)
         gtk_widget_queue_draw_area (tmaid->drawing,
                                 0, (i - tmaid->top.y + 1) * tmaid->font_height,
-                    tmaid->drawing->allocation.width, j * tmaid->font_height);
+                                allocation.width, j * tmaid->font_height);
     }
   if (tmaid->cursor.x < tmaid->top.x)
     tmaid->top.x = tmaid->cursor.x;
@@ -591,13 +602,8 @@ prompt_close (TmaidWindow *tmaid)
       gint result;
       GtkWidget *dialog;
 
-#if GLIB_CHECK_VERSION(2,6,0)
       utf8str = g_filename_display_name
                                     (orz_mdi_get_file (ORZ_MDI (mdi), page));
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-      utf8str = g_filename_to_utf8
-                (orz_mdi_get_file (ORZ_MDI (mdi), page), -1, NULL, NULL, NULL);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
       dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_INFO, GTK_BUTTONS_NONE,
@@ -730,8 +736,10 @@ void
 margin_operation (TmaidWindow *tmaid)
 {
   gboolean limit;
+  GtkAllocation allocation;
   LineBuffer *p;
 
+  gtk_widget_get_allocation (tmaid->drawing, &allocation);
   delete_list (&tmaid->undo);
   delete_list (&tmaid->redo);
   limit = tmaid->ft.limit;
@@ -751,13 +759,11 @@ margin_operation (TmaidWindow *tmaid)
   misc_set_scroll_bar (tmaid->hscroll,
             G_CALLBACK (signal_value_changed_hscroll), tmaid,
             0, edit_get_width_max (tmaid) + 1,
-            MAX (tmaid->drawing->allocation.width / tmaid->font_width, 1),
-                                                                tmaid->top.x);
+            MAX (allocation.width / tmaid->font_width, 1), tmaid->top.x);
   misc_set_scroll_bar (tmaid->vscroll,
             G_CALLBACK (signal_value_changed_vscroll), tmaid,
             0, tmaid->max,
-            MAX (tmaid->drawing->allocation.height / tmaid->font_height, 1),
-                                                                tmaid->top.y);
+            MAX (allocation.height / tmaid->font_height, 1), tmaid->top.y);
   set_menu_bar (tmaid);
   gtk_widget_queue_draw (tmaid->drawing);
 }
