@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -115,10 +115,17 @@ reload_dialog (FileType    *ft,
                             G_CALLBACK (reload_dialog_ok_sensitive), dialog);
   /* ja:コンボボックス */
 #if GTK_CHECK_VERSION(2,4,0)
+# if GTK_CHECK_VERSION(2,24,0)
+  combo = gtk_combo_box_text_new ();
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), "CR+LF");
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), "CR");
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), "LF");
+# else /* not GTK_CHECK_VERSION(2,24,0) */
   combo = gtk_combo_box_new_text ();
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), "CR+LF");
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), "CR");
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), "LF");
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
   gtk_combo_box_set_active (GTK_COMBO_BOX (combo), ft->crlf);
   g_signal_connect (G_OBJECT (combo), "changed",
                             G_CALLBACK (reload_dialog_ok_sensitive), dialog);
@@ -148,15 +155,14 @@ reload_dialog (FileType    *ft,
                             G_CALLBACK (reload_dialog_ok_sensitive), dialog);
   g_object_set_data (G_OBJECT (dialog), "user_data", charset);
   /* ja:フレームとボックス */
-  vbox = gtk_vbox_new (FALSE, SPACING);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
+  vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  gtk_box_set_spacing (GTK_BOX (vbox), SPACING);
   if (text)
     gtk_box_pack_start (GTK_BOX (vbox), gtk_label_new (text), FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), combo, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), charset, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), check0, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), check1, FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
 
   /* ja:表示 */
   gtk_widget_show_all (vbox);
