@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,7 +48,8 @@ signal_timeout (gpointer data)
                 tmaid->cursor.y, FALSE) - tmaid->top.x) * tmaid->font_width,
             (tmaid->cursor.y - tmaid->top.y) * tmaid->font_height,
                                         tmaid->font_width, tmaid->font_height);
-      gdk_window_process_updates (tmaid->drawing->window, TRUE);
+      gdk_window_process_updates (gtk_widget_get_window (tmaid->drawing),
+                                                                        TRUE);
     }
   return TRUE;
 }
@@ -68,7 +69,9 @@ signal_focus_in (GtkWidget     *widget,
   gchar *str;
 #endif /* not G_OS_WIN32 */
 
+#if ! GTK_CHECK_VERSION(2,22,0)
   GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
+#endif /* not GTK_CHECK_VERSION(2,22,0) */
 #ifndef G_OS_WIN32
   gtk_im_context_get_preedit_string (tmaid->im_context, &str, NULL, NULL);
   if (g_strlen (str) > 0)
@@ -88,7 +91,9 @@ signal_focus_out (GtkWidget     *widget,
                   GdkEventFocus *event,
                   TmaidWindow   *tmaid)
 {
+#if ! GTK_CHECK_VERSION(2,22,0)
   GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
+#endif /* not GTK_CHECK_VERSION(2,22,0) */
 #ifndef G_OS_WIN32
   gtk_widget_hide (gtk_widget_get_parent (tmaid->preedit));
 #endif /* not G_OS_WIN32 */
@@ -106,9 +111,10 @@ signal_realize (GtkWidget   *widget,
 
   /* ja:カーソル */
   cursor = gdk_cursor_new (GDK_XTERM);
-  gdk_window_set_cursor (widget->window, cursor);
+  gdk_window_set_cursor (gtk_widget_get_window (widget), cursor);
   gdk_cursor_unref (cursor);
-  gtk_im_context_set_client_window (tmaid->im_context, widget->window);
+  gtk_im_context_set_client_window (tmaid->im_context,
+                                            gtk_widget_get_window (widget));
   /* ja:フォント */
   g_object_unref (tmaid->layout);
   if (tmaid->ft.font_name)
@@ -142,7 +148,7 @@ signal_value_changed_hscroll (GtkAdjustment *adjust,
   GdkPoint top;
 
   top = tmaid->top;
-  tmaid->top.x = adjust->value;
+  tmaid->top.x = gtk_adjustment_get_value (adjust);
   move_edit_window (tmaid, &top);
   draw_caret (tmaid, NULL);
 }
@@ -154,16 +160,18 @@ signal_value_changed_vscroll (GtkAdjustment *adjust,
 {
   gint max, sx;
   GdkPoint top;
+  GtkAllocation allocation;
 
+  gtk_widget_get_allocation (tmaid->drawing, &allocation);
   top = tmaid->top;
-  tmaid->top.y = adjust->value + 0.5;
-  sx = MAX (tmaid->drawing->allocation.width / tmaid->font_width, 1);
+  tmaid->top.y = gtk_adjustment_get_value (adjust) + 0.5;
+  sx = MAX (allocation.width / tmaid->font_width, 1);
   max = edit_get_width_max (tmaid);
   if (tmaid->top.x > max - sx + 1)
     tmaid->top.x = MAX (max - sx + 1, 0);
   move_edit_window (tmaid, &top);
-  if (tmaid->top.x != top.x
-        || gtk_range_get_adjustment (GTK_RANGE (tmaid->hscroll))->upper != max)
+  if (tmaid->top.x != top.x || gtk_adjustment_get_upper
+                (gtk_range_get_adjustment (GTK_RANGE (tmaid->hscroll))) != max)
         misc_set_scroll_bar (tmaid->hscroll,
                             G_CALLBACK (signal_value_changed_hscroll), tmaid,
                                                 0, max + 1, sx, tmaid->top.x);
@@ -359,7 +367,12 @@ signal_expose_preedit (GtkWidget      *widget,
                        TmaidWindow    *tmaid)
 {
   gchar *str;
+#if GTK_CHECK_VERSION(2,24,0)
+  cairo_t *cr;
+#else /* not GTK_CHECK_VERSION(2,24,0) */
   GdkGC *gc;
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
+  GdkWindow *window;
   PangoAttrList *attrs;
 
   gtk_im_context_get_preedit_string (tmaid->im_context, &str, &attrs, NULL);
@@ -368,19 +381,36 @@ signal_expose_preedit (GtkWidget      *widget,
   pango_layout_set_attributes (tmaid->layout, attrs);
   pango_attr_list_unref (attrs);
 
-  gc = gdk_gc_new (widget->window);
+  window = gtk_widget_get_window (widget);
+#if GTK_CHECK_VERSION(2,24,0)
+  cr = gdk_cairo_create (window);
+  /* ja:背景 */
+  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 1);
+  cairo_rectangle (cr, event->area.x, event->area.y,
+                                        event->area.width, event->area.height);
+  cairo_fill (cr);
+  /* ja:文字 */
+  gdk_cairo_set_source_color (cr, tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color);
+  cairo_move_to (cr, 0, 0);
+  pango_cairo_show_layout (cr, tmaid->layout);
+  cairo_destroy (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
+  gc = gdk_gc_new (window);
   /* ja:背景 */
   gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 1);
-  gdk_draw_rectangle (widget->window, gc, TRUE, event->area.x, event->area.y,
+  gdk_draw_rectangle (window, gc, TRUE, event->area.x, event->area.y,
                                         event->area.width, event->area.height);
   /* ja:文字 */
-  gdk_gc_set_foreground (gc, (tmaid->ft.syscol
-                                        ? system_color : tmaid->ft.color));
+  gdk_gc_set_foreground (gc, tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color);
   gdk_gc_set_background (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 1);
-  gdk_draw_layout (widget->window, gc, 0, 0, tmaid->layout);
+  gdk_draw_layout (window, gc, 0, 0, tmaid->layout);
   g_object_unref (gc);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
   pango_layout_set_attributes (tmaid->layout, NULL);
   return TRUE;
 }
@@ -398,13 +428,28 @@ signal_expose (GtkWidget      *widget,
                TmaidWindow    *tmaid)
 {
   gint y;
+
+#if GTK_CHECK_VERSION(2,24,0)
+  cairo_t *cr;
+#else /* not GTK_CHECK_VERSION(2,24,0) */
   GdkGC *gc;
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
   GdkPoint start, end, cursor;
+  GdkWindow *window;
+  GtkAllocation allocation;
   LineBuffer *p;
 
-  gc = gdk_gc_new (widget->window);
+  gtk_widget_get_allocation (widget, &allocation);
+  window = gtk_widget_get_window (widget);
+#if GTK_CHECK_VERSION(2,24,0)
+  cr = gdk_cairo_create (window);
+  cairo_set_line_width (cr, 1);
+  cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
+  gc = gdk_gc_new (window);
   gdk_gc_set_line_attributes (gc, 1,
                                 GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
   cursor.x = edit_get_align_pos (tmaid,
                                    tmaid->cursor.x, tmaid->cursor.y, FALSE);
   cursor.y = tmaid->cursor.y;
@@ -429,50 +474,102 @@ signal_expose (GtkWidget      *widget,
       end = tmaid->select;
     }
   /* ja:背景 */
+#if GTK_CHECK_VERSION(2,24,0)
+  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 1);
+  cairo_rectangle (cr, event->area.x, event->area.y,
+                                        event->area.width, event->area.height);
+  cairo_fill (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
   gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 1);
-  gdk_draw_rectangle (widget->window, gc, TRUE, event->area.x, event->area.y,
+  gdk_draw_rectangle (window, gc, TRUE, event->area.x, event->area.y,
                                         event->area.width, event->area.height);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
   if (tmaid->ft.gline)
     {
       /* ja:グリッド */
       gint x;
 
+#if GTK_CHECK_VERSION(2,24,0)
+      gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 7);
+      for (x = tmaid->font_width - 1; x < allocation.width;
+                                                    x += tmaid->font_width)
+        {
+          cairo_move_to (cr, x, event->area.y);
+          cairo_line_to (cr, x, event->area. y + event->area.height);
+          cairo_stroke (cr);
+        }
+      for (y = tmaid->font_height - 1; y < allocation.height;
+                                                    y += tmaid->font_height)
+        {
+          cairo_move_to (cr, event->area.x, y);
+          cairo_line_to (cr, event->area.x + event->area.width, y);
+          cairo_stroke (cr);
+        }
+#else /* not GTK_CHECK_VERSION(2,24,0) */
       gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 7);
-      for (x = tmaid->font_width - 1; x < widget->allocation.width;
+      for (x = tmaid->font_width - 1; x < allocation.width;
                                                     x += tmaid->font_width)
-        gdk_draw_line (widget->window, gc,
+        gdk_draw_line (window, gc,
                     x, event->area.y, x, event->area. y + event->area.height);
-      for (y = tmaid->font_height - 1; y < widget->allocation.height;
+      for (y = tmaid->font_height - 1; y < allocation.height;
                                                     y += tmaid->font_height)
-        gdk_draw_line (widget->window, gc,
+        gdk_draw_line (window, gc,
                     event->area.x, y, event->area.x + event->area.width, y);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
     }
   if (tmaid->ft.vline)
     {
       /* ja:縦線 */
       gint x;
 
+#if GTK_CHECK_VERSION(2,24,0)
+      gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 7);
+      for (x = (tmaid->ft.tab - tmaid->top.x % tmaid->ft.tab)
+                                                    * tmaid->font_width - 1;
+                                        x < allocation.width;
+                                        x += tmaid->font_width * tmaid->ft.tab)
+        {
+          cairo_move_to (cr, x, event->area.y);
+          cairo_line_to (cr, x, event->area. y + event->area.height);
+          cairo_stroke (cr);
+        }
+#else /* not GTK_CHECK_VERSION(2,24,0) */
       gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 7);
       for (x = (tmaid->ft.tab - tmaid->top.x % tmaid->ft.tab)
                                                     * tmaid->font_width - 1;
-                                        x < widget->allocation.width;
+                                        x < allocation.width;
                                         x += tmaid->font_width * tmaid->ft.tab)
-        gdk_draw_line (widget->window, gc,
+        gdk_draw_line (window, gc,
                     x, event->area.y, x, event->area.y + event->area.height);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
     }
   if (tmaid->ft.mline)
     {
       /* ja:右マージン */
       gint x;
 
+#if GTK_CHECK_VERSION(2,24,0)
+      gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 6);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
       gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 6);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
       x = (tmaid->ft.margin - tmaid->top.x) * tmaid->font_width;
-      gdk_draw_line (widget->window, gc,
+#if GTK_CHECK_VERSION(2,24,0)
+      cairo_move_to (cr, x, event->area.y);
+      cairo_line_to (cr, x, event->area.y + event->area.height);
+      cairo_stroke (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
+      gdk_draw_line (window, gc,
                     x, event->area.y, x, event->area.y + event->area.height);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
     }
 
   for (y = 0, p = edit_get_line_buf (&tmaid->start, &tmaid->off, tmaid->top.y);
@@ -536,28 +633,52 @@ signal_expose (GtkWidget      *widget,
               if (select)
                 {
                   /* ja:選択 */
+#if GTK_CHECK_VERSION(2,24,0)
+                  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 9);
+                  cairo_rectangle (cr, x, y, width, tmaid->font_height);
+                  cairo_fill (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
                   gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 9);
-                  gdk_draw_rectangle (widget->window, gc, TRUE, x, y,
+                  gdk_draw_rectangle (window, gc, TRUE, x, y,
                                                     width, tmaid->font_height);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
                 }
               if (tmaid->ft.uline)
                 {
                   /* ja:下線 */
+#if GTK_CHECK_VERSION(2,24,0)
+                  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 5);
+                  cairo_move_to (cr, x, y + tmaid->font_height - 1);
+                  cairo_line_to (cr, x + width, y + tmaid->font_height - 1);
+                  cairo_stroke (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
                   gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 5);
-                  gdk_draw_line (widget->window, gc,
-                                        x, y + tmaid->font_height - 1, 
+                  gdk_draw_line (window, gc,
+                                        x, y + tmaid->font_height - 1,
                                         x + width, y + tmaid->font_height - 1);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
                 }
               if (tmaid->ft.code)
                 {
                   /* ja:コード文字 */
-                  gdk_gc_set_foreground (gc, (tmaid->ft.syscol
+#if GTK_CHECK_VERSION(2,24,0)
+                  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 2);
-                  gdk_draw_rectangle (widget->window, gc, FALSE,
+                  cairo_rectangle (cr,
                         x + tmaid->font_width / 6, y + tmaid->font_height / 3,
                         tmaid->font_height / 3, tmaid->font_height / 3);
+                  cairo_stroke (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
+                  gdk_gc_set_foreground (gc, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 2);
+                  gdk_draw_rectangle (window, gc, FALSE,
+                        x + tmaid->font_width / 6, y + tmaid->font_height / 3,
+                        tmaid->font_height / 3, tmaid->font_height / 3);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
                 }
               data_pos++;
             }
@@ -572,17 +693,31 @@ signal_expose (GtkWidget      *widget,
               if (select)
                 {
                   /* ja:選択 */
+#if GTK_CHECK_VERSION(2,24,0)
+                  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 9);
+                  cairo_rectangle (cr, x, y, width, tmaid->font_height);
+                  cairo_fill (cr);
+                  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 8);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
                   gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 9);
-                  gdk_draw_rectangle (widget->window, gc, TRUE, x, y,
+                  gdk_draw_rectangle (window, gc, TRUE, x, y,
                                                     width, tmaid->font_height);
                   gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 8);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
                 }
               else
                 {
+#if GTK_CHECK_VERSION(2,24,0)
+                  gdk_cairo_set_source_color (cr, tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
                   gdk_gc_set_foreground (gc, tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
                 }
               if (tmaid->ft.space)
                 {
@@ -598,10 +733,18 @@ signal_expose (GtkWidget      *widget,
               if (space)
                 {
                   /* ja:全角スペース */
+#if GTK_CHECK_VERSION(2,24,0)
+                  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 3);
+                  cairo_rectangle (cr, x + 1, y + 1,
+                                            width - 1, tmaid->font_height - 1);
+                  cairo_fill (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
                   gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 3);
-                  gdk_draw_rectangle (widget->window, gc, TRUE, x + 1, y + 1,
+                  gdk_draw_rectangle (window, gc, TRUE, x + 1, y + 1,
                                             width - 1, tmaid->font_height - 1);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
                 }
               else
                 {
@@ -611,8 +754,13 @@ signal_expose (GtkWidget      *widget,
                   pango_layout_set_text (tmaid->layout, p->text + data_pos,
                                         charset_length (p->text[data_pos]));
                   pango_layout_get_pixel_size (tmaid->layout, &w, NULL);
-                  gdk_draw_layout (widget->window, gc, x + (width - w) / 2, y,
+#if GTK_CHECK_VERSION(2,24,0)
+                  cairo_move_to (cr, x + (width - w) / 2, y);
+                  pango_cairo_show_layout (cr, tmaid->layout);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
+                  gdk_draw_layout (window, gc, x + (width - w) / 2, y,
                                                                 tmaid->layout);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
                 }
               data_pos += charset_length (p->text[data_pos]);
             }
@@ -625,33 +773,64 @@ signal_expose (GtkWidget      *widget,
               if (select)
                 {
                   /* ja:選択 */
+#if GTK_CHECK_VERSION(2,24,0)
+                  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 9);
+                  cairo_rectangle (cr, x, y, width, tmaid->font_height);
+                  cairo_fill (cr);
+                  gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 8);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
                   gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 9);
-                  gdk_draw_rectangle (widget->window, gc, TRUE, x, y,
+                  gdk_draw_rectangle (window, gc, TRUE, x, y,
                                                     width, tmaid->font_height);
                   gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 8);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
                 }
               else
                 {
+#if GTK_CHECK_VERSION(2,24,0)
+                  gdk_cairo_set_source_color (cr, tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
                   gdk_gc_set_foreground (gc, tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
                 }
               pango_layout_set_text (tmaid->layout, p->text + data_pos, 1);
               pango_layout_get_pixel_size (tmaid->layout, &w, NULL);
-              gdk_draw_layout (widget->window, gc, x + (width - w) / 2, y,
+#if GTK_CHECK_VERSION(2,24,0)
+              cairo_move_to (cr, x + (width - w) / 2, y);
+              pango_cairo_show_layout (cr, tmaid->layout);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
+              gdk_draw_layout (window, gc, x + (width - w) / 2, y,
                                                                 tmaid->layout);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
               data_pos++;
             }
           /* ja:キャレットの描画 */
-          if (caret && GTK_WIDGET_HAS_FOCUS (widget)
+          if (caret && gtk_widget_has_focus (widget)
                         && cursor.x == tmaid->top.x + x / tmaid->font_width
                         && cursor.y == tmaid->top.y + y / tmaid->font_height)
             {
+#if GTK_CHECK_VERSION(2,24,0)
+              cairo_operator_t op;
+
+              op = cairo_get_operator (cr);
+              cairo_set_source_rgb (cr, 1, 1, 1);
+              cairo_set_operator (cr, CAIRO_OPERATOR_DIFFERENCE);
+              cairo_rectangle (cr, x, y,
+                            ins ? 2 : tmaid->font_width, tmaid->font_height);
+              cairo_fill (cr);
+              cairo_set_operator (cr, op);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
               gdk_gc_set_function (gc, GDK_INVERT);
-              gdk_draw_rectangle (widget->window, gc, TRUE, x, y,
+              gdk_draw_rectangle (window, gc, TRUE, x, y,
                             ins ? 2 : tmaid->font_width, tmaid->font_height);
               gdk_gc_set_function (gc, GDK_COPY);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
             }
           x += width;
         }
@@ -661,22 +840,45 @@ signal_expose (GtkWidget      *widget,
                             && tmaid->top.y + y / tmaid->font_height < end.y)
             {
               /* ja:選択 */
+#if GTK_CHECK_VERSION(2,24,0)
+              gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 9);
+              cairo_rectangle (cr, x, y,
+                                        tmaid->font_width, tmaid->font_height);
+              cairo_fill (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
               gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 9);
-              gdk_draw_rectangle (widget->window, gc, TRUE, x, y,
+              gdk_draw_rectangle (window, gc, TRUE, x, y,
                                         tmaid->font_width, tmaid->font_height);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
             }
+#if GTK_CHECK_VERSION(2,24,0)
+          gdk_cairo_set_source_color (cr, (tmaid->ft.syscol
+                                        ? system_color : tmaid->ft.color) + 4);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
           gdk_gc_set_foreground (gc, (tmaid->ft.syscol
                                         ? system_color : tmaid->ft.color) + 4);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
           if (!p->next)
             {
               /* ja:ファイルの最後 */
-              gdk_draw_arc (widget->window, gc, TRUE,
+#if GTK_CHECK_VERSION(2,24,0)
+              cairo_arc (cr,
+                        x + tmaid->font_width / 2, y + tmaid->font_height / 2,
+                        tmaid->font_height / 6, 0, G_PI * 2);
+              cairo_fill (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
+              gdk_draw_arc (window, gc, TRUE,
                 x + tmaid->font_width / 6, y + tmaid->font_height / 3,
                 tmaid->font_height / 3, tmaid->font_height / 3, 0, 360 * 64);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
             }
           else
             {
+#if GTK_CHECK_VERSION(2,24,0)
+              gint i;
+#endif /* GTK_CHECK_VERSION(2,8,0) */
               gint length;
               GdkPoint pt[4];
 
@@ -699,22 +901,46 @@ signal_expose (GtkWidget      *widget,
                   pt[2].y = y + tmaid->font_height * 2 / 3;
                   length = 3;
                 }
-              gdk_draw_polygon (widget->window, gc, TRUE, pt, length);
+#if GTK_CHECK_VERSION(2,24,0)
+              cairo_move_to (cr, pt[0].x, pt[0].y);
+              for (i = 1; i < length; i++)
+                cairo_line_to (cr, pt[i].x, pt[i].y);
+              cairo_close_path (cr);
+              cairo_fill (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
+              gdk_draw_polygon (window, gc, TRUE, pt, length);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
             }
         }
       /* ja:キャレットの描画 */
-      if (caret && GTK_WIDGET_HAS_FOCUS (widget)
+      if (caret && gtk_widget_has_focus (widget)
                         && cursor.x == tmaid->top.x + x / tmaid->font_width
                         && cursor.y == tmaid->top.y + y / tmaid->font_height)
         {
+#if GTK_CHECK_VERSION(2,24,0)
+          cairo_operator_t op;
+
+          op = cairo_get_operator (cr);
+          cairo_set_source_rgb (cr, 1, 1, 1);
+          cairo_set_operator (cr, CAIRO_OPERATOR_DIFFERENCE);
+          cairo_rectangle (cr, x, y,
+                            ins ? 2 : tmaid->font_width, tmaid->font_height);
+          cairo_fill (cr);
+          cairo_set_operator (cr, op);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
           gdk_gc_set_function (gc, GDK_INVERT);
-          gdk_draw_rectangle (widget->window, gc, TRUE, x, y,
+          gdk_draw_rectangle (window, gc, TRUE, x, y,
                             ins ? 2 : tmaid->font_width, tmaid->font_height);
           gdk_gc_set_function (gc, GDK_COPY);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
         }
     }
 
+#if GTK_CHECK_VERSION(2,24,0)
+  cairo_destroy (cr);
+#else /* not GTK_CHECK_VERSION(2,24,0) */
   g_object_unref (gc);
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
   return TRUE;
 }
 
@@ -729,7 +955,7 @@ signal_button_press (GtkWidget      *widget,
                      GdkEventButton *event,
                      TmaidWindow    *tmaid)
 {
-  if (!GTK_WIDGET_HAS_FOCUS (widget))
+  if (!gtk_widget_has_focus (widget))
     gtk_widget_grab_focus (widget);
   switch (event->type)
     {
@@ -788,7 +1014,7 @@ signal_button_press (GtkWidget      *widget,
                 TmaidHistory *d;
 
                 if (gdk_selection_owner_get (GDK_SELECTION_PRIMARY)
-                                    == widget->window && tmaid->select.x >= 0)
+                    == gtk_widget_get_window (widget) && tmaid->select.x >= 0)
                   {
                     /* ja:セレクションのオーナーが自分のとき */
                     /* ja:選択範囲を取得 */
@@ -838,7 +1064,11 @@ signal_button_press (GtkWidget      *widget,
                 GtkWidget *menu_shell, *menu_item;
 
                 menu_item = misc_find_menu (popup_entries, "/input");
+#if GTK_CHECK_VERSION(2,12,0)
+                gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), NULL);
+#else /* not GTK_CHECK_VERSION(2,12,0) */
                 gtk_menu_item_remove_submenu (GTK_MENU_ITEM (menu_item));
+#endif /* not GTK_CHECK_VERSION(2,12,0) */
                 menu_shell = gtk_menu_new ();
                 gtk_im_multicontext_append_menuitems (GTK_IM_MULTICONTEXT
                             (tmaid->im_context), GTK_MENU_SHELL (menu_shell));
@@ -899,28 +1129,30 @@ signal_motion_notify_draw (TmaidWindow     *tmaid,
 {
   gint s, sx, sy;
   GdkPoint cursor, select, top, ps;
+  GtkAllocation allocation;
 
   cursor = tmaid->cursor;
   select = tmaid->select;
   top = tmaid->top;
-  sx = MAX (tmaid->drawing->allocation.width / tmaid->font_width, 1);
-  sy = MAX (tmaid->drawing->allocation.height / tmaid->font_height, 1);
+  gtk_widget_get_allocation (tmaid->drawing, &allocation);
+  sx = MAX (allocation.width / tmaid->font_width, 1);
+  sy = MAX (allocation.height / tmaid->font_height, 1);
   ps.x = x;
   ps.y = y;
   if (ps.x < 0)
     ps.x = 0;
-  else if (ps.x > tmaid->drawing->allocation.width)
-    ps.x = tmaid->drawing->allocation.width;
+  else if (ps.x > allocation.width)
+    ps.x = allocation.width;
   if (ps.y < 0)
     ps.y = 0;
-  else if (ps.y > tmaid->drawing->allocation.height)
-    ps.y = tmaid->drawing->allocation.height;
+  else if (ps.y > allocation.height)
+    ps.y = allocation.height;
   /* ja:カーソルが外にあるときスクロールする */
   if (sx > 1)
     {
       if (x < 0 && tmaid->top.x > 0)
         tmaid->top.x--;
-      else if (x > tmaid->drawing->allocation.width
+      else if (x > allocation.width
             && tmaid->top.x < edit_get_width (tmaid, tmaid->cursor.y) + 1 - sx)
         tmaid->top.x++;
     }
@@ -928,8 +1160,7 @@ signal_motion_notify_draw (TmaidWindow     *tmaid,
     {
       if (y < 0 && tmaid->top.y > 0)
         tmaid->top.y--;
-      else if (y > tmaid->drawing->allocation.height
-                                            && tmaid->top.y < tmaid->max - sy)
+      else if (y > allocation.height && tmaid->top.y < tmaid->max - sy)
         tmaid->top.y++;
     }
   s = ps.x - ((tmaid->select.x < 0 ? tmaid->cursor.x : tmaid->select.x)
@@ -992,8 +1223,9 @@ signal_timeout_draw (TmaidWindow *tmaid)
   gint x, y;
   GdkModifierType state;
 
-  gdk_window_get_pointer (tmaid->drawing->window, &x, &y, &state);
-  if (state&GDK_BUTTON1_MASK)
+  gdk_window_get_pointer (gtk_widget_get_window (tmaid->drawing),
+                                                            &x, &y, &state);
+  if (state & GDK_BUTTON1_MASK)
     {
       result = signal_motion_notify_draw (tmaid, x, y, state);
       if (!result)
@@ -1060,6 +1292,7 @@ signal_scroll (GtkWidget      *widget,
 {
   gint max, sx, sy;
   GdkPoint top;
+  GtkAllocation allocation;
 
   top = tmaid->top;
   switch (event->direction)
@@ -1069,8 +1302,9 @@ signal_scroll (GtkWidget      *widget,
       case GDK_SCROLL_LEFT: tmaid->top.x--; break;
       case GDK_SCROLL_RIGHT: tmaid->top.x++;
     }
-  sx = MAX (tmaid->drawing->allocation.width / tmaid->font_width, 1);
-  sy = MAX (tmaid->drawing->allocation.height / tmaid->font_height, 1);
+  gtk_widget_get_allocation (tmaid->drawing, &allocation);
+  sx = MAX (allocation.width / tmaid->font_width, 1);
+  sy = MAX (allocation.height / tmaid->font_height, 1);
   if (tmaid->top.y < 0)
     tmaid->top.y = 0;
   else if (tmaid->top.y > tmaid->max - sy)
@@ -1083,8 +1317,8 @@ signal_scroll (GtkWidget      *widget,
     misc_set_scroll_bar (tmaid->vscroll,
                             G_CALLBACK (signal_value_changed_vscroll), tmaid,
                                             0, tmaid->max, sy, tmaid->top.y);
-  if (tmaid->top.x != top.x
-        || gtk_range_get_adjustment (GTK_RANGE (tmaid->hscroll))->upper != max)
+  if (tmaid->top.x != top.x || gtk_adjustment_get_upper
+                (gtk_range_get_adjustment (GTK_RANGE (tmaid->hscroll))) != max)
     misc_set_scroll_bar (tmaid->hscroll,
                             G_CALLBACK (signal_value_changed_hscroll), tmaid,
                                             0, max + 1, sx, tmaid->top.x);
