@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -53,8 +53,10 @@ signal_style_set (GtkWidget *widget,
   system_color[7] = style->mid[0];
   system_color[8] = style->fg[3];
   system_color[9] = style->bg[3];
+#if ! GTK_CHECK_VERSION(3,0,0)
   gdk_colormap_alloc_colors (gdk_colormap_get_system (),
                                         system_color, 9, FALSE, TRUE, success);
+#endif /* not GTK_CHECK_VERSION(3,0,0) */
   /* ja:システムフォント */
   if (system_font)
     pango_font_description_free (system_font);
@@ -70,6 +72,9 @@ signal_style_set (GtkWidget *widget,
         {
           if (!tmaid->ft.font_name)
             {
+              GtkAllocation allocation;
+
+              gtk_widget_get_allocation (tmaid->drawing, &allocation);
               /* ja:フォント変更 */
               g_object_unref (tmaid->layout);
               gtk_widget_modify_font (tmaid->drawing, system_font);
@@ -82,15 +87,15 @@ signal_style_set (GtkWidget *widget,
                                             = tmaid->top.x = tmaid->top.y = 0;
               modify_margin (tmaid);
               misc_set_scroll_bar (tmaid->hscroll,
-                    G_CALLBACK (signal_value_changed_hscroll), tmaid,
-                    0, edit_get_width_max (tmaid) + 1,
-                    MAX (tmaid->drawing->allocation.width / tmaid->font_width,
-                                                            1), tmaid->top.x);
+                            G_CALLBACK (signal_value_changed_hscroll), tmaid,
+                            0, edit_get_width_max (tmaid) + 1,
+                            MAX (allocation.width / tmaid->font_width, 1),
+                            tmaid->top.x);
               misc_set_scroll_bar (tmaid->vscroll,
-                    G_CALLBACK (signal_value_changed_vscroll), tmaid,
-                    0, tmaid->max,
-                    MAX (tmaid->drawing->allocation.height
-                                    / tmaid->font_height, 1), tmaid->top.y);
+                            G_CALLBACK (signal_value_changed_vscroll), tmaid,
+                            0, tmaid->max,
+                            MAX (allocation.height / tmaid->font_height, 1),
+                            tmaid->top.y);
             }
           gtk_widget_queue_draw (tmaid->drawing);/* ja:再描画 */
         }
@@ -154,24 +159,31 @@ signal_drag_data_received (GtkWidget        *widget,
                            GdkDragContext   *context,
                            gint              x,
                            gint              y,
-                           GtkSelectionData *selection_data,
+                           GtkSelectionData *data,
                            guint             info,
                            guint             time)
 {
-  gchar *file, **files;
-  gint i;
+  gchar **files;
 
-  files = g_strsplit ((const gchar *)selection_data->data, "\r\n", 0);
-  for (i = 0; files[i] != NULL; i++)
+  files = g_strsplit ((const gchar *)gtk_selection_data_get_data (data),
+                                                                    "\r\n", 0);
+  if (files)
     {
-      file = g_filename_from_uri (files[i], NULL, NULL);
-      if (file)
+      gint i;
+
+      for (i = 0; files[i]; i++)
         {
-          file_open_edit (file);
-          g_free (file);
+          gchar *file;
+
+          file = g_filename_from_uri (files[i], NULL, NULL);
+          if (file)
+            {
+              file_open_edit (file);
+              g_free (file);
+            }
         }
+      g_strfreev (files);
     }
-  g_strfreev (files);
 }
 
 
@@ -197,7 +209,8 @@ signal_selection_get (GtkWidget        *widget,
                       guint             time,
                       TmaidWindow      *tmaid)
 {
-  if (data->selection==GDK_SELECTION_PRIMARY && tmaid && tmaid->select.x >= 0)
+  if (gtk_selection_data_get_selection (data) == GDK_SELECTION_PRIMARY
+                                            && tmaid && tmaid->select.x >= 0)
     /* ja:プライマリセレクション */
     {
       gchar *text;
@@ -210,7 +223,8 @@ signal_selection_get (GtkWidget        *widget,
       gtk_selection_data_set_text (data, text, length);
       g_free (text);
     }
-  else if (data->selection == GDK_SELECTION_CLIPBOARD && clipboard_text)
+  else if (gtk_selection_data_get_selection (data) == GDK_SELECTION_CLIPBOARD
+                                                            && clipboard_text)
     {
       /* ja:クリップボード */
       gtk_selection_data_set_text (data,
@@ -225,21 +239,22 @@ signal_selection_received (GtkWidget        *widget,
                            guint             time,
                            gpointer          user_data)
 {
-  if (data->selection == GDK_SELECTION_PRIMARY
-                                || data->selection == GDK_SELECTION_CLIPBOARD)
+  if (gtk_selection_data_get_selection (data) == GDK_SELECTION_PRIMARY
+        || gtk_selection_data_get_selection (data) == GDK_SELECTION_CLIPBOARD)
     {
       gchar *text = NULL;
 
       /* ja:プライマリセレクション/クリップボード */
-      if (data->length >= 0)
+      if (gtk_selection_data_get_length (data) >= 0)
         {
-          if (data->type == GDK_SELECTION_TYPE_ATOM)
+          if (gtk_selection_data_get_data_type (data)
+                                                    == GDK_SELECTION_TYPE_ATOM)
             {
               gint i,count;
               GdkAtom *atoms;
 
-              atoms = (GdkAtom *)data->data;
-              count = data->length / sizeof (GdkAtom);
+              atoms = (GdkAtom *)gtk_selection_data_get_data (data);
+              count = gtk_selection_data_get_length (data) / sizeof (GdkAtom);
               for (i = 0; i < count; i++)
                 if (atoms[i] == atom_utf8)
                   break;
@@ -252,8 +267,9 @@ signal_selection_received (GtkWidget        *widget,
                   if (atoms[i] == GDK_TARGET_STRING)
                     break;
               if (i < count)
-                gtk_selection_convert (widget, data->selection, atoms[i],
-                                                            GDK_CURRENT_TIME);
+                gtk_selection_convert (widget,
+                                    gtk_selection_data_get_selection (data),
+                                                atoms[i], GDK_CURRENT_TIME);
             }
           else
             {
@@ -271,7 +287,8 @@ signal_selection_received (GtkWidget        *widget,
           tmaid = orz_mdi_get_data (ORZ_MDI (mdi), page_num);
           select = tmaid->select;
           d = edit_operation (tmaid, text, g_strlen (text),
-                            data->selection == GDK_SELECTION_CLIPBOARD, FALSE);
+            gtk_selection_data_get_selection (data) == GDK_SELECTION_CLIPBOARD,
+                                                                        FALSE);
           d->next = tmaid->undo;
           tmaid->undo = d;
           if (delete_list (&tmaid->redo) > 0 || !d->next || select.x >= 0)
