@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -126,7 +126,9 @@ main (int   argc,
 
   /* ja:初期化 */
   setlocale (LC_ALL, "");
+#if ! GTK_CHECK_VERSION(3,0,0)
   gtk_set_locale ();
+#endif /* not GTK_CHECK_VERSION(3,0,0) */
   gtk_init (&argc, &argv);
   bindtextdomain (PACKAGE, misc_spec_locale (argv[0]));
   bind_textdomain_codeset (PACKAGE, "UTF-8");
@@ -254,7 +256,7 @@ TMAID_COPYRIGHT_NOTICE"\n"
                                 G_CALLBACK (signal_close_window), NULL);
   g_signal_connect (G_OBJECT (mdi), "switch-window",
                                 G_CALLBACK (signal_switch_window), NULL);
-  g_signal_connect (GTK_OBJECT (mdi), "destroy",
+  g_signal_connect (G_OBJECT (mdi), "destroy",
                                 G_CALLBACK (signal_destroy_mdi), NULL);
   /* ja:ポップアップ */
   accel_group = gtk_accel_group_new ();
@@ -360,8 +362,10 @@ TMAID_COPYRIGHT_NOTICE"\n"
   system_color[7] = style->mid[0];
   system_color[8] = style->fg[3];
   system_color[9] = style->bg[3];
+#if ! GTK_CHECK_VERSION(3,0,0)
   gdk_colormap_alloc_colors (gdk_colormap_get_system (),
                                     system_color, 12, FALSE, TRUE, success);
+#endif /* not GTK_CHECK_VERSION(3,0,0) */
   /* ja:システムフォント */
   system_font = pango_font_description_copy
                                     (pango_context_get_font_description
