@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2010 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -102,8 +102,8 @@ const static gchar *authors =
                                 gtk_label_new (_("Authors & Contributors")));
 
   /* ja:フレームとボックス */
-  vbox = gtk_vbox_new (FALSE, SPACING);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
+  vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  gtk_box_set_spacing (GTK_BOX (vbox), SPACING);
   hbox0 = gtk_hbox_new (FALSE, SPACING);
   pixbuf = gdk_pixbuf_new_from_xpm_data (tmaid32_xpm);
   gtk_box_pack_start (GTK_BOX (hbox0),
@@ -116,7 +116,6 @@ const static gchar *authors =
   gtk_box_pack_start (GTK_BOX (hbox1), hbox0, TRUE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
 
   /* ja:表示 */
   gtk_widget_show_all (vbox);
